@@ -1367,11 +1367,32 @@ pidgin_notify_uri(const char *uri)
 			command = g_strdup_printf("opera %s", escaped);
 
 	}
+	else if (!strcmp(web_browser, "google-chrome"))
+	{
+		/* Google Chrome doesn't have command-line arguments that control the
+		 * opening of links from external calls.  This is controlled solely from
+		 * a preference within Google Chrome. */
+		command = g_strdup_printf("google-chrome %s", escaped);
+	}
+	else if (!strcmp(web_browser, "chrome"))
+	{
+		/* Chromium doesn't have command-line arguments that control the
+		 * opening of links from external calls.  This is controlled solely from
+		 * a preference within Chromium. */
+		command = g_strdup_printf("chrome %s", escaped);
+	}
+	else if (!strcmp(web_browser, "chromium-browser"))
+	{
+		/* Chromium doesn't have command-line arguments that control the
+		 * opening of links from external calls.  This is controlled solely from
+		 * a preference within Chromium. */
+		command = g_strdup_printf("chromium-browser %s", escaped);
+	}
 	else if (!strcmp(web_browser, "custom"))
 	{
 		const char *web_command;
 
-		web_command = purple_prefs_get_path(PIDGIN_PREFS_ROOT "/browsers/command");
+		web_command = purple_prefs_get_string(PIDGIN_PREFS_ROOT "/browsers/manual_command");
 
 		if (web_command == NULL || *web_command == '\0')
 		{
@@ -1571,8 +1592,15 @@ pidgin_create_notification_dialog(PidginNotifyType type)
 		gtk_widget_set_sensitive(button, FALSE);
 		spec_dialog->edit_button = button;
 
-		button = gtk_dialog_add_button(GTK_DIALOG(dialog),
-						_("Dismiss"), GTK_RESPONSE_NO);
+		/* Translators: Make sure you translate "Dismiss" differently than
+		   "close"!  This string is used in the "You have pounced" dialog
+		   that appears when one of your Buddy Pounces is triggered.  In
+		   this context "Dismiss" means "I acknowledge that I've seen that
+		   this pounce was triggered--remove it from this list."  Translating
+		   it as "Remove" is acceptable if you can't think of a more precise
+		   word. */
+		button = gtk_dialog_add_button(GTK_DIALOG(dialog), _("Dismiss"),
+				GTK_RESPONSE_NO);
 		gtk_widget_set_sensitive(button, FALSE);
 		spec_dialog->dismiss_button = button;
 
